@@ -27,11 +27,13 @@
 #include"red_black_tree.h"
 #include<stdio.h>
 #include<ctype.h>
-#include<math.h>
+#include<cmath>
+#include<algorithm>
 
 using namespace Rcpp;
 
 // [[Rcpp::depends(RcppArmadillo)]]
+// [[Rcpp::interfaces(r, cpp)]]
 
 // The following functions are used as input to initalize a red black tree whose
 // nodes contain doubles. Knowing why these functions exist is likely
@@ -72,9 +74,227 @@ void bubbleSort(long double * vec, int n) {
 }
 
 /***
- * The main function implementing the fast computation of the t* U-statistic
+ * For an input vector x = (x_1,...,x_n) returns a new vector r of length n
+ * with r[i] equal to the rank of x_i in x. So if x == (4,2,2,3,2,3) then
+ * r == (3, 1, 1, 2, 1, 2).
+ * @param x an arma::vec to get the ranks of
+ * @return the vector of ranks
+ */
+arma::uvec vecToRanks(const arma::vec& x) {
+  if (x.n_elem == 0) {
+    return(arma::zeros<arma::uvec>(0));
+  }
+  arma::uvec xSortedInds = arma::sort_index(x);
+  arma::uvec ranks = arma::zeros<arma::uvec>(x.n_elem);
+  arma::uvec repeats = arma::zeros<arma::uvec>(x.n_elem);
+  int k = 1;
+  double last = x[xSortedInds[0]];
+  for (int i = 0; i < x.n_elem; i++) {
+    double current = x[xSortedInds[i]];
+    if (current != last) {
+      k++;
+      last = current;
+    }
+    repeats[i] = k;
+  }
+  for (int i = 0; i < xSortedInds.n_elem; i++) {
+    ranks[xSortedInds[i]] = repeats[i];
+  }
+  return ranks;
+}
+
+/***
+ * For two input vectors of ranks xRanks, yRanks (i.e. taking values in the
+ * positive integers and including all integers between 1 and their maximum
+ * value) returns the matrix A where A[i,j] is the number times and element of
+ * xRanks, and the corresponding element of yRanks, are less than or equal to i
+ * and j respectively. Hence (xRanks[s],yRanks[s]) contributes to A[i,j] if and
+ * only if x[s] <= i and y[s] <= i.
+ * @param xRanks a vector of ranks.
+ * @param yRanks a vector of ranks (must be the same length as x).
+ * @return the "less than or equal" to matrix.
+ */
+arma::umat ranksToLeqMat(const arma::uvec& xRanks, const arma::uvec& yRanks) {
+  int xMax = xRanks.max();
+  int yMax = yRanks.max();
+  arma::umat leqMat = arma::zeros<arma::umat>(xMax + 1, yMax + 1);
+  for (int i = 0; i < xRanks.n_elem; i++) {
+    leqMat(xRanks[i], yRanks[i]) += 1;
+  }
+  for (int i = 1; i <= xMax; i++) {
+    for (int j = 1; j <= yMax; j++) {
+      leqMat(i,j) =
+        leqMat(i,j - 1) + leqMat(i - 1,j) - leqMat(i - 1, j - 1) + leqMat(i,j);
+    }
+  }
+  return leqMat;
+}
+
+/***
+ * Computes a version of the matrix defined at the top of page 5 of Heller and
+ * Heller (2016) (arxiv:1605.08732). In particular, we let the rank by always
+ * <= rather than < in the x case. This makes the notation nice in
+ * TStarHellerAndHellerRCPP.
+ * @param leqMat a leqMat as created by ranksToLeqMat.
+ * @return the unique count matrix.
+ */
+arma::umat leqMatToUniqueCountMat(const arma::umat& leqMat) {
+  arma::umat uCountMat = arma::zeros<arma::umat>(leqMat.n_rows, leqMat.n_cols);
+  for (int i = 1; i < leqMat.n_rows; i++) {
+    for (int j = 1; j < leqMat.n_cols; j++) {
+      int numEqYAndLessOrEqX = leqMat(i, j) - leqMat(i, j - 1);
+      uCountMat(i,j) = uCountMat(i, j - 1) +
+        (numEqYAndLessOrEqX * (numEqYAndLessOrEqX - 1)) / 2;
+    }
+  }
+  return uCountMat;
+}
+
+/***
+ * Given an arma::uvec x and a set of indices inds, creates a new vector whose
+ * entries are those of x indexed by inds. So if
+ * x = (1,2,3)
+ * inds = (0,0,2)
+ * then we would return the vector (1,1,3).
+ * @param x vector be be indexed into
+ * @param inds the indices
+ * @return a new vector.
+ */
+arma::uvec indexUvec(const arma::uvec& x, const arma::uvec& inds) {
+  arma::uvec newVec = arma::zeros<arma::uvec>(inds.n_elem);
+  for (int i = 0; i < newVec.n_elem; i++) {
+    newVec[i] = x[inds[i]];
+  }
+  return newVec;
+}
+
+/***
+ * Function implementing the O(n^2) computation of the t* U-statistic
  * described in
  *
+ * Heller, Yair and Heller, Ruth. "Computing the Bergsma Dassios sign-
+ * covariance." arXiv preprint arXiv:1605.08732 (2016).
+ *
+ * @param x a arma::vec of values
+ * @param y a arma::vec of values of the same length as x
+ * @return the U-statistic computed upon the two input vectors.
+ */
+// [[Rcpp::export]]
+double TStarHellerAndHellerRCPP(const arma::vec& x, const arma::vec& y) {
+  arma::uvec xRanks = vecToRanks(x);
+  arma::uvec yRanks = vecToRanks(y);
+  arma::umat leqMat = ranksToLeqMat(xRanks, yRanks);
+  arma::uvec xOrder = arma::sort_index(xRanks);
+  xRanks = indexUvec(xRanks, xOrder);
+  yRanks = indexUvec(yRanks, xOrder);
+
+  arma::umat uCountMat = leqMatToUniqueCountMat(leqMat);
+
+  double numCon = 0;
+  double numDis = 0;
+  for (int i = 0; i < xRanks.n_elem - 1; i++) {
+    for (int j = i + 1; j < xRanks.n_elem; j++) {
+      int xRankMin = xRanks[i];
+      int yRankMin = std::min(yRanks[i], yRanks[j]);
+      int yRankMax = std::max(yRanks[i], yRanks[j]);
+
+      int bot = leqMat(xRankMin - 1, yRankMin - 1);
+      int mid = (yRankMin == yRankMax) ? 0 :
+        leqMat(xRankMin - 1, yRankMax - 1) - leqMat(xRankMin - 1, yRankMin);
+      int top = leqMat(xRankMin - 1, leqMat.n_cols - 1) -
+                 leqMat(xRankMin - 1, yRankMax);
+      int eqMin = leqMat(xRankMin - 1, yRankMin) -
+        leqMat(xRankMin - 1, yRankMin - 1);
+      int eqMax = leqMat(xRankMin - 1, yRankMax) -
+        leqMat(xRankMin - 1, yRankMax - 1);
+
+      numCon += top * (top - 1) / 2.0 + bot * (bot - 1) / 2.0;
+
+      if (yRankMin != yRankMax) {
+        numDis += top * (mid + eqMin + bot) + bot * (mid + eqMax) +
+          eqMin * (mid + eqMax) +
+          eqMax * mid + mid * (mid - 1) / 2.0;
+        numDis -= uCountMat(xRankMin - 1, yRankMax - 1) -
+          uCountMat(xRankMin - 1, yRankMin);
+      }
+    }
+  }
+
+  int n = xRanks.n_elem;
+  double c = 16 * numCon - 8 * numDis;
+  double d = (c < 0) ? -1 : 1;
+  return d * expl(logl(d * c) - (logl(n) + logl(n - 1) +
+                               logl(n - 2) + logl(n - 3)));
+}
+
+/***
+ * Function implementing the O(n^2) computation of the t* V-statistic
+ * using a modified version of the algorithm used in TStarHellerAndHellerRCPP.
+ *
+ * Heller, Yair and Heller, Ruth. "Computing the Bergsma Dassios sign-
+ * covariance." arXiv preprint arXiv:1605.08732 (2016).
+ *
+ * @param x a arma::vec of values
+ * @param y a arma::vec of values of the same length as x
+ * @return the V-statistic computed upon the two input vectors.
+ */
+// [[Rcpp::export]]
+double VTStarHellerAndHellerRCPP(const arma::vec& x, const arma::vec& y) {
+  arma::uvec xRanks = vecToRanks(x);
+  arma::uvec yRanks = vecToRanks(y);
+  arma::umat leqMat = ranksToLeqMat(xRanks, yRanks);
+  arma::uvec xOrder = arma::sort_index(xRanks);
+  xRanks = indexUvec(xRanks, xOrder);
+  yRanks = indexUvec(yRanks, xOrder);
+
+  arma::umat uCountMat = leqMatToUniqueCountMat(leqMat);
+
+  double numCon = 0;
+  double numDis = 0;
+  for (int i = 0; i < xRanks.n_elem; i++) {
+    int top = leqMat(xRanks[i] - 1, leqMat.n_cols - 1) -
+      leqMat(xRanks[i] - 1, yRanks[i]);
+    int bot = leqMat(xRanks[i] - 1, yRanks[i] - 1);
+    numCon += 0.5 * (top * (top - 1) / 2.0 + bot * (bot - 1) / 2.0) +
+      0.25 * (top + bot);
+
+    for (int j = i + 1; j < xRanks.n_elem; j++) {
+      int xRankMin = xRanks[i];
+      int yRankMin = std::min(yRanks[i], yRanks[j]);
+      int yRankMax = std::max(yRanks[i], yRanks[j]);
+
+      int bot = leqMat(xRankMin - 1, yRankMin - 1);
+      int mid = (yRankMin == yRankMax) ? 0 :
+        leqMat(xRankMin - 1, yRankMax - 1) - leqMat(xRankMin - 1, yRankMin);
+      int top = leqMat(xRankMin - 1, leqMat.n_cols - 1) -
+        leqMat(xRankMin - 1, yRankMax);
+      int eqMin = leqMat(xRankMin - 1, yRankMin) -
+        leqMat(xRankMin - 1, yRankMin - 1);
+      int eqMax = leqMat(xRankMin - 1, yRankMax) -
+        leqMat(xRankMin - 1, yRankMax - 1);
+
+      numCon += top * (top - 1) / 2.0 + bot * (bot - 1) / 2.0 + 0.5 * (top + bot);
+
+      if (yRankMin != yRankMax) {
+        numDis += top * (mid + eqMin + bot) + bot * (mid + eqMax) +
+          eqMin * (mid + eqMax) +
+          eqMax * mid + mid * (mid - 1) / 2.0;
+        numDis -= uCountMat(xRankMin - 1, yRankMax - 1) -
+          uCountMat(xRankMin - 1, yRankMin);
+      }
+    }
+  }
+
+  int n = xRanks.n_elem;
+  double c = 16 * numCon - 8 * numDis;
+  double d = (c < 0) ? -1 : 1;
+  return d * expl(logl(d * c) - 4 * logl(n));
+}
+
+/***
+ * The main function implementing the O(n^2*log(n)) computation of the t*
+ * U-statistic described in
+ *
  * Weihs, Luca, Mathias Drton, and Dennis Leung. "Efficient Computation of the
  * Bergsma-Dassios Sign Covariance." arXiv preprint arXiv:1504.00964 (2015).
  *
@@ -86,7 +306,7 @@ void bubbleSort(long double * vec, int n) {
  * @return the U-statistic computed upon the two input vectors.
  */
 // [[Rcpp::export]]
-Rcpp::NumericVector TStarFastTiesRCPP(NumericVector xNumeric,
+Rcpp::NumericVector TStarWeihsEtAlRCPP(NumericVector xNumeric,
                                       NumericVector yNumeric) {
 	// Copy input data to something understood by the rbtree functions
   int n = xNumeric.size();
@@ -135,8 +355,8 @@ Rcpp::NumericVector TStarFastTiesRCPP(NumericVector xNumeric,
     // equations from the paper to record the number of concordant/discordant
     // quadruples.
 		for (int j = i + 1; j < n; j++) {
-			double minY = fmin(y[i], y[j]);
-			double maxY = fmax(y[i], y[j]);
+			double minY = std::min(y[i], y[j]);
+			double maxY = std::max(y[i], y[j]);
 
 			int numLessMax = RBNumLessThan(tree, &maxY);
 			int numGreaterMin = RBNumGreaterThan(tree, &minY);
@@ -178,8 +398,8 @@ Rcpp::NumericVector TStarFastTiesRCPP(NumericVector xNumeric,
 			savedYsInds[0] = i;
 		}
 		for(int j = i - 1; j >= 0; j--) {
-			double minY = fmin(y[i], y[j]);
-			double maxY = fmax(y[i], y[j]);
+			double minY = std::min(y[i], y[j]);
+			double maxY = std::max(y[i], y[j]);
 			if(minY == maxY) {
 				discord -= RBNumGreaterThan(revTree, &maxY) *
           RBNumLessThan(revTree, &maxY);
@@ -202,7 +422,7 @@ Rcpp::NumericVector TStarFastTiesRCPP(NumericVector xNumeric,
 }
 
 /***
- * The same as TStarFastTiesRCPP except computes the V-statistic form of the
+ * The same as TStarWeihsEtAlRCPP except computes the V-statistic form of the
  * statistic.
  *
  * @param xNumeric a NumericVector of values
@@ -210,11 +430,11 @@ Rcpp::NumericVector TStarFastTiesRCPP(NumericVector xNumeric,
  * @return the V-statistic computed upon the two input vectors.
  */
 // [[Rcpp::export]]
-Rcpp::NumericVector VTStarFastTiesRCPP(NumericVector xNumeric,
+Rcpp::NumericVector VTStarWeihsEtAlRCPP(NumericVector xNumeric,
                                        NumericVector yNumeric) {
   /***
    * Note, there are very few inline comments here as they would be almost
-   * identical to those comments in the TStarFastTiesRCPP function. See the
+   * identical to those comments in the TStarWeihsEtAlRCPP function. See the
    * paper for differences in the two algorithms.
    */
 	// Copy input data to something understood by the rbtree functions
@@ -253,8 +473,8 @@ Rcpp::NumericVector VTStarFastTiesRCPP(NumericVector xNumeric,
 		concord += .5 * ((bottom * (bottom - 1)) / 2) + .5 * ((top * (top - 1)) / 2)
       + .25 * (top + bottom);
 		for(int j = i + 1; j < n; j++) {
-			double minY = fmin(y[i], y[j]);
-			double maxY = fmax(y[i], y[j]);
+			double minY = std::min(y[i], y[j]);
+			double maxY = std::max(y[i], y[j]);
 
 			int numLessMax = RBNumLessThan(tree, &maxY);
 			int numGreaterMin = RBNumGreaterThan(tree, &minY);
@@ -298,8 +518,8 @@ Rcpp::NumericVector VTStarFastTiesRCPP(NumericVector xNumeric,
 			savedYsInds[0] = i;
 		}
 		for(int j = i - 1; j >= 0; j--) {
-			double minY = fmin(y[i], y[j]);
-			double maxY = fmax(y[i], y[j]);
+			double minY = std::min(y[i], y[j]);
+			double maxY = std::max(y[i], y[j]);
 			if(minY == maxY) {
 				discord -= RBNumGreaterThan(revTree, &maxY) * RBNumLessThan(revTree, &maxY);
 			}
@@ -339,7 +559,7 @@ IntegerVector integerSort(IntegerVector x) {
 
 /***
  * Estimates tau* by iteratively resampling small subsets of the data, computing
- * t* (see VTStarFastTiesRCPP) across these subsets, and then averaging the
+ * t* (see VTStarWeihsEtAlRCPP) across these subsets, and then averaging the
  * result.
  *
  * @param xNumeric a NumericVector of values
@@ -363,13 +583,13 @@ Rcpp::NumericVector TStarFastResampleRCPP(NumericVector xNumeric,
   IntegerVector inds;
   for(int i = 0; i < numResamples; i++) {
       inds = integerSort(RcppArmadillo::sample(allIndices, sampleSize, false, NumericVector::create()));
-      toReturn[0] += TStarFastTiesRCPP(
+      toReturn[0] += TStarHellerAndHellerRCPP(
         getSubset(xNumeric, inds),
         getSubset(yNumeric, inds)
-      )[0];
+      );
   }
   toReturn[0] = toReturn[0] / numResamples;
-  return(toReturn);
+  return toReturn;
 }
 
 /***
@@ -381,26 +601,26 @@ Rcpp::NumericVector TStarFastResampleRCPP(NumericVector xNumeric,
  */
 int bergDassAFunc(double z1, double z2, double z3, double z4) {
   int returnVal = 0;
-  returnVal += (fmax(z1, z3) < fmin(z2, z4)) ? 1 : 0;
-  returnVal += (fmax(z2, z4) < fmin(z1, z3)) ? 1 : 0;
-  returnVal += (fmax(z1, z2) < fmin(z3, z4)) ? -1 : 0;
-  returnVal += (fmax(z3, z4) < fmin(z1, z2)) ? -1 : 0;
+  returnVal += (std::max(z1, z3) < std::min(z2, z4)) ? 1 : 0;
+  returnVal += (std::max(z2, z4) < std::min(z1, z3)) ? 1 : 0;
+  returnVal += (std::max(z1, z2) < std::min(z3, z4)) ? -1 : 0;
+  returnVal += (std::max(z3, z4) < std::min(z1, z2)) ? -1 : 0;
   return returnVal;
 }
 
 /***
- * Similar to TStarFastTiesRCPP but implements a slow, naive algorithm for the
+ * Similar to TStarWeihsEtAlRCPP but implements a slow, naive algorithm for the
  * computation of the U/V-statistics. This allows for the comparision of the old
  * algorithm to the new.
  *
- * @param x see xNumeric from TStarFastTiesRCPP
- * @param y see yNumeric from TStarFastTiesRCPP
+ * @param x see xNumeric from TStarWeihsEtAlRCPP
+ * @param y see yNumeric from TStarWeihsEtAlRCPP
  * @param vStat a boolean that, if true, will result in the computation of the
  *        V-statistic form of t* and, if false, will result in the computation
  *        of the U-statistic form of t*
  */
 // [[Rcpp::export]]
-Rcpp::NumericVector TStarSlowTiesRCPP(NumericVector x, NumericVector y, bool vStat) {
+Rcpp::NumericVector TStarNaiveRCPP(NumericVector x, NumericVector y, bool vStat) {
   int i,j,k,l;
   long double sumVal = 0;
 	long double z[4];
@@ -429,11 +649,11 @@ Rcpp::NumericVector TStarSlowTiesRCPP(NumericVector x, NumericVector y, bool vSt
   					z[0] = y[i]; z[1] = y[j]; z[2] = y[k]; z[3] = y[l];
   					bubbleSort(z, 4);
 
-  					if ((fmax(x[a], x[b]) < fmin(x[c], x[d])) &&
-  						 ((fmax(y[a], y[b]) < fmin(y[c], y[d])) ||
-  							(fmin(y[a], y[b]) > fmax(y[c], y[d])))) {
+  					if ((std::max(x[a], x[b]) < std::min(x[c], x[d])) &&
+  						 ((std::max(y[a], y[b]) < std::min(y[c], y[d])) ||
+  							(std::min(y[a], y[b]) > std::max(y[c], y[d])))) {
   						sumVal += 16;
-  					} else if ((fmax(x[a], x[b]) < fmin(x[c], x[d])) && (z[1] < z[2])) {
+  					} else if ((std::max(x[a], x[b]) < std::min(x[c], x[d])) && (z[1] < z[2])) {
   						sumVal -= 8;
   					}
   				}
